#include "stdafx.h"
#include "CListCtrlEx.h"
#include "Utility.h"

using namespace std;

const vector<string> vecEmpty;

CListCtrlEx* CListCtrlEx::m_pThis = NULL;
int iSortDirection = 1;

int CALLBACK CListCtrlEx::CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CListCtrlEx* pListCtrl = (CListCtrlEx*) m_pThis;
	CString strItem1 = pListCtrl->GetItemText((int)lParam1, (int)lParamSort);
	CString strItem2 = pListCtrl->GetItemText((int)lParam2, (int)lParamSort);
	return strcmp(strItem2, strItem1) * iSortDirection;
}

void CListCtrlEx::SetHeader(const std::vector<std::string>& header)
{
	this->DeleteAllItems();
	for (int i = (int)m_vecHeader.size() - 1; i >= 0; --i){
		this->DeleteColumn(i);
	}
	for (int column = 0; column < (int)header.size(); ++column){
		this->InsertColumn(column, header[column].c_str());
	}
	m_vecHeader = header;
	m_vecSortDirection.clear();
	m_vecSortDirection.resize(m_vecHeader.size(), 1);
}

int CListCtrlEx::InsertItem(const std::vector<std::string>& item, int index)
{
	if (m_vecHeader.size() != item.size()){
		return -1;
	}
	int line = (index < 0) ? this->GetItemCount() : index;
	CListCtrl::InsertItem(line, item[0].c_str());
	for (int column = 1; column < (int)item.size(); ++column){
		SetItemText(line, column, item[column].c_str());
	}
	return line;
}

const std::vector<std::string> CListCtrlEx::GetItem(int index)
{
	if (this->GetItemCount() <= index){
		return vecEmpty;
	}
	vector<string> res;
	res.reserve(m_vecHeader.size());
	for (int column = 0; column < (int)m_vecHeader.size(); ++column){
		res.push_back((LPCTSTR)this->GetItemText(index, column));
	}
	return res;
}

int CListCtrlEx::GetSortDirection(int index)
{
	return m_vecSortDirection[index] *= -1;
}

bool CListCtrlEx::SortItem(int column)
{
	if ((int)m_vecHeader.size() <= column){
		return false;
	}
	m_pThis = this;
	LV_ITEM lvitem;
	ZeroMemory(&lvitem, sizeof(lvitem));
	for( int i = 0; i < this->GetItemCount(); i++ ) {
		lvitem.iItem = i;
		lvitem.iSubItem = 0;
		lvitem.mask = LVIF_PARAM;
		lvitem.lParam = i;
		this->SetItem(&lvitem);
	}
	iSortDirection = m_pThis->GetSortDirection(column);
	return SortItems(CListCtrlEx::CompareFunc, column) == TRUE;
}

bool CListCtrlEx::SortItem(int column, int direction)
{
	if ((int)m_vecHeader.size() <= column){
		return false;
	}
	m_vecSortDirection[column] = direction * -1;
	return SortItem(column);
}

long long CListCtrlEx::GetItemInt(int row, int column)
{
	return (int)Utility::ToInt((LPCTSTR)this->GetItemText(row, column));
}
