#include "stdafx.h"
#include <afxinet.h>
#include "InternetFile.h"
#include "Log.h"
#include "Utility.h"

using namespace std;

const int InternetFile::HTTP_BUFFER_SIZE = 64 * 1024;

InternetFile::InternetFile(CInternetSession* session, const std::string& url) : m_pServer(NULL), m_pFile(NULL), m_bIsOpened(false){
	try {
		DWORD protocol = 0;
		CString serverName;
		CString fileName;
		INTERNET_PORT port;
		if (AfxParseURL(url.c_str(), protocol, serverName, fileName, port) == false)
		{
			Log::Add("InternetFile::InternetFile() : URLp[XG[ : " + url);
			return;
		}

		m_pServer = session->GetHttpConnection(serverName, port);
		m_pFile = m_pServer->OpenRequest(CHttpConnection::HTTP_VERB_GET, fileName);
		ASSERT(m_pFile->SetReadBufferSize(HTTP_BUFFER_SIZE));
		m_pFile->SendRequest();
		DWORD dwRet;
		m_pFile->QueryInfoStatusCode(dwRet);

		if (dwRet == HTTP_STATUS_OK)
		{
			m_bIsOpened = true;
		}

	} catch (CInternetException* pEx) {
		//catch errors from WinInet
		char buffer[1024];
		pEx->GetErrorMessage(buffer, 1023);
		Log::Add("InternetFile::InternetFile() : C^[lbgڑOLb`");
		Log::Add(buffer);
	}
}

InternetFile::~InternetFile(){
	if (m_pFile){
		m_pFile->Close();
		delete m_pFile;
		m_pFile = NULL;
	}
	if (m_pServer){
		m_pServer->Close();
		delete m_pServer;
		m_pServer = NULL;
	}
}

bool InternetFile::ReadString(std::string& s){
	try {
		CString temp_string;
		if (m_pFile->ReadString(temp_string)){
			s = Utility::EUCtoSJIS((LPCTSTR)temp_string);
			return true;
		}
	} catch (CInternetException* pEx) {
		//catch errors from WinInet
		char buffer[1024];
		pEx->GetErrorMessage(buffer, 1023);
		Log::Add("InternetFile::ReadString() : C^[lbgڑOLb`");
		Log::Add(buffer);
	}
	return false;
}

int InternetFile::Read(void* buffer, int size)
{
	try {
		return m_pFile->Read(buffer, size);
	} catch (CInternetException* pEx) {
		//catch errors from WinInet
		char buffer[1024];
		pEx->GetErrorMessage(buffer, 1023);
		Log::Add("InternetFile::ReadString() : C^[lbgڑOLb`");
		Log::Add(buffer);
	}
	return 0;
}

const std::string InternetFile::GetFileName()const
{
	return (LPCTSTR)m_pFile->GetFileName();
}
