#include "stdafx.h"
#include "Utility.h"
#include <cstdio>
#include <cstdlib>
#include <sstream>
#include <boost/crc.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>
#include <afx.h>
#include "Log.h"

#ifdef USE_CHAR_CODE
#include "CharCode.h"
#endif

using namespace std;

string strCurrentFolder;
vector<string> vecFindFile;
char cByteTable[256];

void Utility::Initialize()
{
	srand((unsigned int)time(NULL));
	char buffer[MAX_PATH];
	::GetModuleFileName(NULL, buffer, MAX_PATH);
	strCurrentFolder = Utility::SplitFileFolder(buffer);
//	string current_folder = buffer;
//	current_folder.erase(current_folder.find_last_of('\\') + 1);
//	strCurrentFolder = current_folder;

	for (int i = 0; i < 256; ++i){
		cByteTable[i] = -1;
	}
	for (char c = '0'; c <= '9'; ++c){
		cByteTable[(int)c] = c - '0';
	}
	for (char c = 'a'; c <= 'f'; ++c){
		cByteTable[(int)c] = c - 'a' + 10;
	}
	for (char c = 'A'; c <= 'F'; ++c){
		cByteTable[(int)c] = c - 'A' + 10;
	}
	setlocale(LC_ALL, "Japanese");
	::SetCurrentDirectory(strCurrentFolder.c_str());
}

void FindFileEx(const std::string& folder, const std::string& file_name, bool sub_folder)
{
	if (sub_folder){
		string wild_card = folder;
		if (wild_card[wild_card.size() - 1] != '\\'){
			wild_card += "\\";
		}
		wild_card += "*.*";
		CFileFind finder_folder;
		BOOL working = finder_folder.FindFile(wild_card.c_str());
		while (working){
			working = finder_folder.FindNextFile();
			if (finder_folder.IsDots()){
				continue;
			}
			if (finder_folder.IsDirectory()){
				string next = (LPCTSTR)finder_folder.GetFilePath();
				FindFileEx(next, file_name, sub_folder);
			}
		}
		finder_folder.Close();
	}

	string wild_card;
	wild_card = folder;
	if (wild_card[wild_card.size() - 1] != '\\'){
		wild_card += "\\";
	}
	wild_card += file_name;
	CFileFind finder_file;
	BOOL working = finder_file.FindFile(wild_card.c_str());
	while (working){
		working = finder_file.FindNextFile();
		if (finder_file.IsDots()){
			continue;
		}
		if (finder_file.IsDirectory()){
			continue;
		}
		vecFindFile.push_back((LPCTSTR)finder_file.GetFilePath());

	}
	finder_file.Close();
}

const vector<string>& Utility::FindFile(const std::string& folder, const std::string& file_name, bool sub_folder)
{
	vecFindFile.clear();
	FindFileEx(folder, file_name, sub_folder);
	return vecFindFile;
}

const string& Utility::GetProgramFolder()
{
	return strCurrentFolder;
}

const std::string Utility::GetCurrentFolder()
{
	const iBufferSize = 1024;
	char buffer[iBufferSize + 1];
	ZeroMemory(buffer, sizeof(buffer));
	::GetCurrentDirectory(iBufferSize, buffer);
	return string(buffer) + "\\";
}

int Utility::GetClusterSize(const std::string& drive)
{
	DWORD secter_per_cluster, bytes_per_sector, number_of_free_clusters, total_number_of_cluseters;
	::GetDiskFreeSpace(((drive.empty()) ? NULL : drive.c_str()), &secter_per_cluster, &bytes_per_sector, &number_of_free_clusters, &total_number_of_cluseters);
	return (int)(secter_per_cluster * bytes_per_sector);
}

bool Utility::IsExist(const std::string& file_path)
{
	CFileStatus status;
	return CFile::GetStatus(file_path.c_str(), status) == TRUE;
}

void Utility::MessageBox(const std::string& message)
{
	AfxMessageBox(message.c_str());
}

//GetFolderNameFromWindow R[obN֐
int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lParam,LPARAM lpData)
{
    if(uMsg==BFFM_INITIALIZED){
        SendMessage(hwnd,BFFM_SETSELECTION,(WPARAM)TRUE,lpData);
    }
    return 0;
}

const std::string Utility::GetFolderNameFromWindow(HWND hwnd, const std::string& folder_path)
{
	LPMALLOC g_pMalloc;

	// IMallocC^tF[X̎擾
	SHGetMalloc(&g_pMalloc);

	BROWSEINFO  bi;
	ZeroMemory(&bi, sizeof(BROWSEINFO));
	char buffer[MAX_PATH];
	bi.hwndOwner        = hwnd;
	bi.pidlRoot         = NULL;
	bi.pszDisplayName   = buffer;
	bi.lpszTitle        = "tH_IĂ";
	bi.ulFlags          = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
	bi.lpfn             = &BrowseCallbackProc;
	bi.lParam           = (LPARAM)folder_path.c_str();
	bi.iImage           = 0;
	//tH_̎Qƃ_CAO\
	string res;
	ITEMIDLIST  *idl = SHBrowseForFolder(&bi);
	if(idl != NULL)
	{
		SHGetPathFromIDList(idl, buffer);
		g_pMalloc->Release();
		res = buffer;
	}
	CoTaskMemFree(idl);
	return res;
}

const string Utility::GetCRC(const std::string& file_path)
{
	FILE *file = fopen(file_path.c_str(), "rbS");
	if (file == NULL){
		return "FFFFFFFF";
	}
	setvbuf(file, NULL, _IOFBF, Utility::GetClusterSize(file_path.substr(0, 2)));
	fseek(file, 0, SEEK_END);
	int file_size = (int)ftell(file);
	fseek(file, 0, SEEK_SET);
	auto_ptr<char> buffer = auto_ptr<char>(new char[file_size]);
	if ((int)fread(buffer.get(), 1, file_size, file) != file_size){
		fclose(file);
		file = NULL;
		return "FFFFFFFF";
	}
	fclose(file);
	file = NULL;
	return GetCRCBuffer(buffer.get(), file_size);
}

const string Utility::GetCRCBuffer(const char* ptr, unsigned int size)
{
	boost::crc_32_type boost_crc;
	boost_crc.process_bytes(ptr, size);
	unsigned int crc = boost_crc.checksum();
	return ToHex(crc, 8);
}

const string Utility::ToLower(const string& in)
{
	auto_ptr<char> buffer(new char[in.size() + 1]);
	transform(in.begin(), in.end(), buffer.get(), &tolower);
	buffer.get()[in.size()] = '\0';
	return string(buffer.get());
	/*
	string res(in);
	for (string::iterator i = res.begin(); i != res.end(); i++){
		*i = (char)tolower(*i);
	}
	return res;
	*/
}

const string Utility::ToUpper(const string& in)
{
	auto_ptr<char> buffer(new char[in.size() + 1]);
	transform(in.begin(), in.end(), buffer.get(), &toupper);
	buffer.get()[in.size()] = '\0';
	return buffer.get();
/*
	string res(in);
	for (string::iterator i = res.begin(); i != res.end(); i++){
		*i = (char)toupper(*i);
	}
	return res;
*/
}

const std::string Utility::GenerateBackupFileName(const std::string& file_path)
{
	for (int i = 0; i < 1000; ++i){
		string new_file_path = file_path + "." + Utility::ToString(i, 3);
		if (!Utility::IsExist(new_file_path)){
			return new_file_path;
		}
	}
	return "";
}

bool Utility::IsFile(const string& file_path)
{
	CFileStatus status;
	if (!CFile::GetStatus(file_path.c_str(), status)){
		return false;
	}
	return (!(status.m_attribute & CFile::Attribute::directory)) ? true : false;
}

bool Utility::IsFolder(const string& file_path)
{
	CFileStatus status;
	if (!CFile::GetStatus(file_path.c_str(), status)){
		return false;
	}
	return (status.m_attribute & CFile::Attribute::directory) ? true : false;
}

vector<string> vecDeleteObject;
void DeleteFolderRef(const std::string& folder_path)
{
	CFileFind finder;
	BOOL working = finder.FindFile((folder_path + "\\*.*").c_str());
	while (working){
		working = finder.FindNextFile();
		if (finder.IsDots()){
			continue;
		}
		if (finder.IsDirectory()){
			DeleteFolderRef((LPCTSTR)finder.GetFilePath());
			continue;
		}
		vecDeleteObject.push_back((LPCTSTR)finder.GetFilePath());
	}
	finder.Close();
	vecDeleteObject.push_back(folder_path);
}

void Utility::DeleteFolder(const std::string& folder_path)
{
	vecDeleteObject.clear();
	DeleteFolderRef(folder_path);
	for (vector<string>::iterator it = vecDeleteObject.begin(); it != vecDeleteObject.end(); ++it){
		if (Utility::IsFile(*it)){
			if (!::DeleteFile(it->c_str())){
				Log::Add("t@C폜ł܂ł " + *it);
				Log::LastError(GetLastError());
			}
		} else if (Utility::IsFolder(*it)){
			if (!::RemoveDirectory(it->c_str())){
				Log::Add("tH_폜ł܂ł " + *it);
				Log::LastError(GetLastError());
			}
		}
	}
	vecDeleteObject.clear();
}

const string Utility::GetRootFolder(const std::string& folder_path)
{
	wstring work = StringToWString(folder_path);
	if (work.find_last_of(L"\\") == 0){
		return folder_path;
	}
	work.erase(work.find_last_of(L"\\"));
	return WStringToString(work);
}

const vector<char> vecEmpty;

const vector<char> Utility::ConvertByteString(const std::string& str)
{
	if (str.size() % 2 == 1){
		return vecEmpty;
	}
	for (string::const_iterator it = str.begin(); it != str.end(); ++it){
		if (cByteTable[(int)*it] < 0){
			return vecEmpty;
		}
	}
	size_t index = 0;
	vector<char> res;
	res.reserve(str.size() / 2);
	while (index != str.size()){
		res.push_back((cByteTable[str[index]] << 4) + cByteTable[str[index + 1]]);
		index += 2;
	}
	return res;
}

long long Utility::GetFileSize(const std::string& file_path)
{
	if (!Utility::IsFile(file_path)){
		return -1;
	}
	return (long long)CFile(file_path.c_str(), CFile::modeRead | CFile::shareDenyNone).GetLength();
}

// l<=>ϊn
const std::string Utility::ToHex(long long number, int digit)
{
	string f = string("%");
	if (digit != 0){
		f += "0";
		f += ToString(digit);
	}
	f += "x";
	return (boost::format(f) % number).str();
}

long long Utility::ToInt(const std::string& str)
{
	for (string::const_iterator it = str.begin(); it != str.end(); ++it){
		if (!isdigit(*it)){
			return -1;
		}
	}
	return boost::lexical_cast<long long>(str);
}

const std::string Utility::ToString(long long number, int digit)
{
	string f = "%";
	if (digit != 0){
		f += "0";
		f += ToString(digit);
	}
	f += "d";
	return (boost::format(f) % number).str();
}

long long Utility::HexToInt(const std::string& hex)
{
	if (hex.size() > 16){
		return -1;
	}
	long long res = 0;
	for (string::const_iterator it = hex.begin(); it != hex.end(); ++it){
		if (cByteTable[*it] < -1){
			return -1;
		}
		res = (res << 4) | (long long)cByteTable[*it];
	}
	return res;
}

const std::string Utility::ToHalf(const std::string& in)
{
	const iBufferSize = (int)(in.size() * 2);
	auto_ptr<char> buffer(new char[iBufferSize]);
	ZeroMemory(buffer.get(), iBufferSize);
	LCMapString(MAKELCID(MAKELANGID(LANG_JAPANESE, SUBLANG_DEFAULT), SORT_DEFAULT), LCMAP_HALFWIDTH, in.c_str(), -1, buffer.get(), (int)in.size() * 2);
	return buffer.get();
}

const std::string Utility::ToFull(const std::string& in)
{
	const iBufferSize = (int)(in.size() * 3);
	auto_ptr<char> buffer(new char[iBufferSize]);
	ZeroMemory(buffer.get(), iBufferSize);
	LCMapString(MAKELCID(MAKELANGID(LANG_JAPANESE, SUBLANG_DEFAULT), SORT_DEFAULT), LCMAP_FULLWIDTH, in.c_str(), -1, buffer.get(), (int)in.size() * 2);
	return buffer.get();
}

const iBufferSize = 64 * 1024;

const std::vector<std::string> Utility::GetFileNameFromWindow(bool is_open, bool is_multi, const std::string& extention, const std::string& init_file_name, int flag, const std::string& filter, CWnd* wnd)
{
	auto_ptr<char> buffer;
	BOOL bOpenFileDialog = is_open ? TRUE : FALSE;
	LPCTSTR lpszDefExt = extention.empty() ? NULL : extention.c_str();
	LPCTSTR lpszFileName = init_file_name.empty() ? NULL : init_file_name.c_str();
	DWORD dwFlags = flag;
	if (is_multi){
		buffer = auto_ptr<TCHAR>(new TCHAR[iBufferSize]);
		ZeroMemory(buffer.get(), iBufferSize * sizeof(TCHAR));
		flag |= OFN_ALLOWMULTISELECT;
	}
	LPCTSTR lpszFilter = filter.empty() ? NULL : filter.c_str();
	CWnd* pParentWnd = wnd;
	CFileDialog dialog(bOpenFileDialog, lpszDefExt, lpszFileName, dwFlags, lpszFilter, pParentWnd);
	if (is_multi){
		dialog.GetOFN().lpstrFile = buffer.get();
		dialog.GetOFN().nMaxFile = iBufferSize;
		dialog.GetOFN().Flags |= OFN_ALLOWMULTISELECT;
	}
	vector<string> res;
	if (dialog.DoModal() == IDOK){
		POSITION pos = dialog.GetStartPosition();
		while (pos != NULL){
			res.push_back((LPCTSTR)dialog.GetNextPathName(pos));
		}
	}
	return res;
}

const std::string Utility::WStringToString(const std::wstring& in)
{
	int size = WideCharToMultiByte(932, 0, in.c_str(), -1, NULL, 0, NULL, NULL) + 1;
	auto_ptr<char> buffer = auto_ptr<char>(new char[size]);
	ZeroMemory(buffer.get(), size * sizeof(char));
	WideCharToMultiByte(CP_ACP, 0, in.c_str(), -1, buffer.get(), size, NULL, NULL);
	return string(buffer.get());
}

const std::wstring Utility::StringToWString(const std::string& in)
{
	int size = MultiByteToWideChar(932, 0, in.c_str(), -1, NULL, 0) + 1;
	auto_ptr<WCHAR> buffer = auto_ptr<WCHAR>(new WCHAR[size]);
	ZeroMemory(buffer.get(), size * sizeof(WCHAR));
	MultiByteToWideChar(CP_ACP, 0, in.c_str(), -1, buffer.get(), size);
	return wstring(buffer.get());
}

std::string& Utility::FilterNonPrintableChar(std::string& in)
{
	string temp;
	for (string::iterator it = in.begin(); it != in.end(); ++it){
		if (isprint(*it)){
			temp.push_back(*it);
		}
	}
	swap(temp, in);
	return in;
}

const std::string Utility::SplitFileTitle(const std::string& file_path)
{
	char buffer_fname[_MAX_FNAME * 2];
	memset(buffer_fname, 0, sizeof(buffer_fname));
	_splitpath(file_path.c_str(), NULL, NULL, buffer_fname, NULL);
	return buffer_fname;
}

const std::string Utility::SplitFileExtention(const std::string& file_path)
{
	char buffer_ext[_MAX_EXT * 2];
	memset(buffer_ext, 0, sizeof(buffer_ext));
	_splitpath(file_path.c_str(), NULL, NULL, NULL, buffer_ext);
	string res = ToLower(buffer_ext);
	if (res != ""){
		return res.substr(1);
	}
	return "";
}

const std::string Utility::SplitFileFolder(const std::string& file_path)
{
	char buffer_drive[_MAX_DRIVE * 2];
	char buffer_dir[_MAX_DIR * 2];
	memset(buffer_drive, 0, sizeof(buffer_drive));
	memset(buffer_dir, 0, sizeof(buffer_dir));
	_splitpath(file_path.c_str(), buffer_drive, buffer_dir, NULL, NULL);
	return string(buffer_drive) + string(buffer_dir);
}

int Utility::Square2(int i)
{
	int bit= 0;
	--i;
	while (i){
		i /= 2;
		++bit;
	}
	return (1 << bit);
}

// PoCgڂJ^Jiۂ𔻒f(JIS)
bool Utility::IsJIS_Kana(char ch)
{
	return ch == 0x8e ? true : false;
}

// PoCgڂۂ𔻒f(JIS)
bool Utility::IsJIS_Kanji(char ch)
{
	return (ch >=0x21 && ch <=0x7e) ? TRUE:FALSE;
}

const std::string Utility::EUCtoSJIS(const std::string& str)
{
#ifdef USE_CHAR_CODE
	return CharCode::EUCtoSJIS(str);
#else
	string res;
	for (int i = 0; i < (int)str.size(); ++i){
		char ch1 = str[i];
		// PoCgڂpJ^Jȉꍇ
		if (IsJIS_Kana(ch1)){
			++i;
			res += str[i];
		} else if (IsJIS_Kanji(ch1 ^ 0x80)){
			++i;
			ch1 ^= 0x80;
			char ch2 = str[i] ^ 0x80;
			WORD wcode= MAKEWORD(ch2, ch1);
			wcode = _mbcjistojms(wcode);
			res += HIBYTE(wcode);
			res += LOBYTE(wcode);
		} else {
			res += ch1;
		}
	}
	return res;
#endif
}

// PoCgڂۂ𔻒f
bool Utility::IsSJIS_Kanji(char ch)
{
    // 0x81 - 0x9F  0xE0 - 0xFC ͈̔
    return _ismbblead( (WORD)ch) != 0 ? TRUE:FALSE;
}

// PoCgڂpJ^Jiۂ𔻒f
bool Utility::IsSJIS_Kana(char ch)
{
    // 0xA1 - 0xDF ͈̔
    return _ismbbkana( (WORD)ch) != 0 ? TRUE:FALSE;
}

// SHIFT-JISEUCɕϊ
const std::string Utility::SJIStoEUC(const std::string& str)
{
#ifdef USE_CHAR_CODE
	return CharCode::SJIStoEUC(str);
#else
	string res;
	for (int i = 0; i < (int)str.size(); ++i){
		char ch1 = str[i];
		if (IsSJIS_Kanji(ch1)){
			char ch2 = str[++i];
			WORD wcode = MAKEWORD(ch2, ch1);
			wcode = _mbcjmstojis(wcode);
			ch1 = HIBYTE(wcode) | 0x80;
			ch2 = LOBYTE(wcode) | 0x80;
			res += ch1;
			res += ch2;
		} else if (IsSJIS_Kana(ch1)){
			res += (char)0x8e;
			res += ch1;
		} else {
			res += ch1;
		}
	}
	return res;
#endif
}
