#include "stdafx.h"
#include <sstream>
#include <boost/regex.hpp>
#include "GameTitleGetter.h"
#include "Control.h"
#include "SettingFile.h"
#include "Resource.h"
#include "memccppDlg.h"
#include "Log.h"
#include "Utility.h"
#include "InternetFile.h"

using namespace std;

boost::regex regexDirectoryMatch("B[IAE][A-Z]{4}[\\-P][0-9]{5}.*");

GameTitleGetter::GameTitleGetter(const std::string& directory, int disp_index) : m_iDispIndex(disp_index)
{
	if (!boost::regex_match(directory, regexDirectoryMatch)){
		return;
	}
	for (int index = 2; index < 6; ++index){
		m_strProductID.push_back(tolower(directory[index]));
	}
	m_strProductID += directory.substr(7, 5);
	//m_hThread = NULL;
}

GameTitleGetter::~GameTitleGetter()
{
	m_Mutex.Lock(1000);
	m_Mutex.Unlock();
	/*
	if (m_hThread){
		WaitForSingleObject(m_hThread, 1000);
		m_hThread = NULL;
	}
	*/
}

bool GameTitleGetter::Run()
{
	if (m_strProductID.empty()){
		return true;
	}
	SettingFile& soft = *Control::GetSoftData();
	if (soft.IsInSection(m_strProductID)){
		FromLocal();
		return true;
	}
#ifdef USE_THREAD
	//CWinThread *thread = AfxBeginThread(GameTitleGetter::Thread, this, THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED);
	CWinThread *thread = AfxBeginThread(GameTitleGetter::Thread, this);
	//DuplicateHandle(GetCurrentProcess(), thread->m_hThread, GetCurrentProcess(), &m_hThread, 0, TRUE, DUPLICATE_SAME_ACCESS);
	//ResumeThread(thread->m_hThread);
	return false;
#else
	Thread(this);
	return true;
#endif
}

UINT GameTitleGetter::Thread(LPVOID pparam)
{
	((GameTitleGetter*)pparam)->Process();
	return 0;
}

void GameTitleGetter::Process()
{
	m_Mutex.Lock();
	Control::GetDialog().SetText(TEXT_GAME_DATA, "Q[擾ł");
	FromInternet();
	m_Mutex.Unlock();
}

const string strKeyTitle = "TITLE";
const string strKeyFormat = "FORMAT";
const string strKeyGenre = "GENRE";
const string strKeyMaker = "MAKER";
const string strKeyPrice = "PRICE";
const string strKeyPriceTaxIn = "PRICE_TAX_IN";
const string strKeySaleDate = "SALE_DATE";

void GameTitleGetter::FromLocal()const
{
	SettingFile& soft = *Control::GetSoftData();
	string text;
	text += Utility::ToHalf(soft.Get(m_strProductID, strKeyTitle));
	text += "\r\n";
	text += Utility::ToHalf(soft.Get(m_strProductID, strKeyGenre));
	text += " ";
	text += Utility::ToHalf(soft.Get(m_strProductID, strKeySaleDate));
	text += "\r\n";
	text += Utility::ToHalf(soft.Get(m_strProductID, strKeyMaker));
	DispGameData(text);
}

const char *strURLPrefix[] = {
	"http://www.playstation.jp/products/title/",
	"http://www.playstation.jp.nyud.net:8090/products/title/"
};

const iHttpBufferSize = 64 * 1024;
void GameTitleGetter::FromInternet()const
{
	if (!Control::GetDialog().GetCheckBoxBool(CHECK_BOX_USE_INTERNET)){
		return;
	}

	const int index_url = Control::GetDialog().GetCheckBoxBool(CHECK_BOX_USE_PROXY) ? 1 : 0;
	const string url = string(strURLPrefix[index_url]) + m_strProductID + ".html";
	InternetFile file(Control::GetInternetSession(), url);
	if (!file.IsOpened()){
		Log::Add("GameTitleGetter::FromInternet() : html擾ł܂ł");
		return;
	}

	SettingFile& soft = *Control::GetSoftData();
	SkipHtml("^Cg", file);

	string value;

	//^Cg
	GetNextString(file, value);

	if (value == "tH[}bg"){
		//NO DATA
		value = "NO DATA";
		soft.Add(m_strProductID, strKeyTitle, value);
		//value = "";
		soft.Add(m_strProductID, strKeyMaker, value);
		soft.Add(m_strProductID, strKeyFormat, value);
		soft.Add(m_strProductID, strKeyGenre, value);
		soft.Add(m_strProductID, strKeyPrice, value);
		soft.Add(m_strProductID, strKeyPriceTaxIn, value);
		soft.Add(m_strProductID, strKeySaleDate, value);
	} else {
		soft.Add(m_strProductID, strKeyTitle, value);

		//[J[
		GetNextString(file, value);
		soft.Add(m_strProductID, strKeyMaker, value);

		//tH[}bg
		GetNextString(file, value);
		GetNextString(file, value);
		soft.Add(m_strProductID, strKeyFormat, value);

		//W
		GetNextString(file, value);
		GetNextString(file, value);
		GetNextString(file, value);
		GetNextString(file, value);
		soft.Add(m_strProductID, strKeyGenre, value);

		//]i
		GetNextString(file, value);
		GetNextString(file, value);
		if (value.find('&') != string::npos){
			value.erase(value.find('&'), string::npos);
		}
		if (value.find(',') != string::npos){
			value.erase(value.find(','), 1);
		}
		soft.Add(m_strProductID, strKeyPrice, value);
		soft.Add(m_strProductID, strKeyPriceTaxIn, Utility::ToString((Utility::ToInt(value) * 105 / 100)));

		//
		GetNextString(file, value);
		GetNextString(file, value);
		soft.Add(m_strProductID, strKeySaleDate, value);
	}

	SetCurrentDirectory(Utility::GetProgramFolder().c_str());
	soft.Save(strSoftDataFileName);
	FromLocal();

/*	I
	CHttpConnection* pServer = NULL;
	CHttpFile *pFile = NULL;
	try
	{
//		INTERNET_PORT nPort = 80;
		int index_url = Control::GetDialog().GetCheckBoxBool(CHECK_BOX_USE_PROXY) ? 1 : 0;
		const string url = string(strURLPrefix[index_url]) + m_strProductID + ".html";

		DWORD protocol = 0;
		CString serverName;
		CString fileName;
		INTERNET_PORT port;
		if (AfxParseURL(url.c_str(), protocol, serverName, fileName, port) == false)
		{
			DispGameData("Q[^Cg擾ł܂\r\n" + m_strProductID);
			Log::Add("URLp[XG[ : " + url);
			return;
		}

		pServer = Control::GetInternetSession()->GetHttpConnection(serverName, port);
		pFile = pServer->OpenRequest(CHttpConnection::HTTP_VERB_GET, fileName);
		ASSERT(pFile->SetReadBufferSize(iHttpBufferSize));
		pFile->SendRequest();
		DWORD dwRet;
		pFile->QueryInfoStatusCode(dwRet);

		if (dwRet == HTTP_STATUS_OK)
		{
			SettingFile& soft = *Control::GetSoftData();

			SkipHtml(Utility::SJIStoEUC("^Cg"), pFile);

			string value;

			//^Cg
			GetNextString(pFile, value);

			if (value == "tH[}bg"){
				//NO DATA
				value = "NO DATA";
				soft.Add(m_strProductID, strKeyTitle, value);
				//value = "";
				soft.Add(m_strProductID, strKeyMaker, value);
				soft.Add(m_strProductID, strKeyFormat, value);
				soft.Add(m_strProductID, strKeyGenre, value);
				soft.Add(m_strProductID, strKeyPrice, value);
				soft.Add(m_strProductID, strKeyPriceTaxIn, value);
				soft.Add(m_strProductID, strKeySaleDate, value);
			} else {
				soft.Add(m_strProductID, strKeyTitle, value);

				//[J[
				GetNextString(pFile, value);
				soft.Add(m_strProductID, strKeyMaker, value);

				//tH[}bg
				GetNextString(pFile, value);
				GetNextString(pFile, value);
				soft.Add(m_strProductID, strKeyFormat, value);

				//W
				GetNextString(pFile, value);
				GetNextString(pFile, value);
				GetNextString(pFile, value);
				GetNextString(pFile, value);
				soft.Add(m_strProductID, strKeyGenre, value);

				//]i
				GetNextString(pFile, value);
				GetNextString(pFile, value);
				if (value.find('&') != string::npos){
					value.erase(value.find('&'), string::npos);
				}
				if (value.find(',') != string::npos){
					value.erase(value.find(','), 1);
				}
				soft.Add(m_strProductID, strKeyPrice, value);
				soft.Add(m_strProductID, strKeyPriceTaxIn, Utility::ToString((Utility::ToInt(value) * 105 / 100)));

				//
				GetNextString(pFile, value);
				GetNextString(pFile, value);
				soft.Add(m_strProductID, strKeySaleDate, value);
			}

			SetCurrentDirectory(Utility::GetProgramFolder().c_str());
			soft.Save(strSoftDataFileName);
			FromLocal();
		} else {
			DispGameData("Q[^Cg擾ł܂\r\n" + m_strProductID);
		}
		pFile->Close();
		delete pFile;
		pServer->Close();
		delete pServer;
	} catch (CInternetException* pEx) {
		//catch errors from WinInet
		DispGameData("Q[^Cg擾ł܂\r\n" + m_strProductID);
		char buffer[1024];
		pEx->GetErrorMessage(buffer, 1023);
		Log::Add(string("C^[lbgڑOLb`\r\n") + buffer);
	}
*/
}

void GameTitleGetter::DispGameData(const std::string& text)const
{
	if (Control::IsExit()){
		return;
	}
	if (Control::GetGameDataDispIndex() < m_iDispIndex){
		Control::SetGameDataDispIndex(m_iDispIndex);
		Control::GetDialog().SetText(TEXT_GAME_DATA, text);
	}
}

bool GameTitleGetter::SkipHtml(const std::string& target, InternetFile& file, int additional_line)const
{
	string s;
	while (file.ReadString(s)){
		if (s.find(target) != string::npos){
			for (int i = 0; i < additional_line; ++i){
				file.ReadString(s);
			}
			return true;
		}
	}
	return false;
}

const string strTagTD = "</TD>";

bool GameTitleGetter::GetNextString(InternetFile& file, std::string& res)const
{
	string s;
	while (file.ReadString(s)){
		wstring line = Utility::StringToWString(s);
		while (!line.empty() && line[0] == L'<'){
			line.erase(0, line.find(L'>') + 1);
		}
		if (line.empty() || line.find_first_not_of(L' ') == string::npos){
			continue;
		}
		wstring temp;
		getline(wistringstream(line), temp, L'<');
		res = Utility::WStringToString(temp);
		return true;
	}
	return false;
}
