#include "stdafx.h"
#include "IconObject.h"
#include "IconSys.h"
#include "IconRender.h"
#include "IconFile.h"
#include "IconTexture.h"

using namespace std;

const D3DXVECTOR4 BackGroundBase[] = {
	D3DXVECTOR4(-1.0f,  1.0f, 1.0f, 1.0f),
	D3DXVECTOR4( 1.0f,  1.0f, 1.0f, 1.0f),
	D3DXVECTOR4(-1.0f, -1.0f, 1.0f, 1.0f),
	D3DXVECTOR4( 1.0f, -1.0f, 1.0f, 1.0f),
};
const int iVertexTurn[] = {0, 1, 2, 1, 2, 3};
const DWORD D3DFVF_BACK_VERTEX =  (D3DFVF_XYZRHW | D3DFVF_DIFFUSE);
const DWORD D3DFVF_ICON_VERTEX = (D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1);
const string strIconFileName = "tempicon.dds";

IconObject::IconObject(const std::string& icon_sys_path)
{
	m_bIsOpen = false;
	IconSys icon_sys(icon_sys_path);
	if (!icon_sys.IsOpen()){
		return;
	}
	for (int i = 0; i < 6; ++i){
		IconObject::BACK_VERTEX v;
		v.pos = BackGroundBase[iVertexTurn[i]];
		v.color = icon_sys.GetBackgroundColor(iVertexTurn[i]);
	}
	for (int i = 0; i < 3; ++i){
		m_LightDirection[i] = icon_sys.GetLightDirection(i);
	}
	for (int i = 0; i < 4; ++i){
		m_LightColor[i] = icon_sys.GetLightColor(i);
	}
	const string icon_file_path = icon_sys_path.substr(0, icon_sys_path.find_last_of('\\') + 1) + icon_sys.GetIconFileName(0);
	IconFile icon_file(icon_file_path);
	if (!icon_file.IsOpen()){
		return;
	}

	const vector<IconFile::VERTEX>& vec = icon_file.GetVertex();
	m_vecVertex.clear();
	m_vecVertex.reserve(vec.size());
	for (int i = 0; i < (int)vec.size(); ++i){
		IconObject::ICON_VERTEX vertex;
		vertex.pos = vec[i].coordinate[0] * -1.0f;
		vertex.normal = vec[i].normal * -1.0f;
		vertex.color = vec[i].color;
		vertex.tex = vec[i].texture_coordinate;
		m_vecVertex.push_back(vertex);
	}
	m_pTexture = boost::shared_ptr<IconTexture>(new IconTexture(icon_file.GetTextureData(), 128, 128));
	DeleteFile(strIconFileName.c_str());
	m_bIsOpen = true;
}

const IconObject::ICON_VERTEX BillBoardVertex[] = {
	IconObject::ICON_VERTEX(D3DXVECTOR3( 2.0f, 4.25f, 0.0f), D3DXVECTOR3(0.0f, 0.0f, 4.0f), 0x808080ff, D3DXVECTOR2(0.0f, 0.0f)),
	IconObject::ICON_VERTEX(D3DXVECTOR3(-2.0f, 4.25f, 0.0f), D3DXVECTOR3(0.0f, 0.0f, 4.0f), 0x808080ff, D3DXVECTOR2(1.0f, 0.0f)),
	IconObject::ICON_VERTEX(D3DXVECTOR3( 2.0f, 0.25f, 0.0f), D3DXVECTOR3(0.0f, 0.0f, 4.0f), 0x808080ff, D3DXVECTOR2(0.0f, 1.0f)),
	IconObject::ICON_VERTEX(D3DXVECTOR3(-2.0f, 0.25f, 0.0f), D3DXVECTOR3(0.0f, 0.0f, 4.0f), 0x808080ff, D3DXVECTOR2(1.0f, 1.0f))
};

const int iBillBoardVertexIndex[] = {0, 2, 1, 2, 3, 1};

IconObject::IconObject(const boost::shared_ptr<IconTexture> icon_texture)
{
	int vertex_count = sizeof(iBillBoardVertexIndex) / sizeof(int);
	m_vecVertex.reserve(vertex_count);
	for (int i = 0; i < vertex_count; ++i){
		m_vecVertex.push_back(BillBoardVertex[iBillBoardVertexIndex[i]]);
	}
	m_pTexture = icon_texture;
}

void IconObject::Draw()const
{
	//wi`
/*
	IconRender::GetDevice()->SetFVF(D3DFVF_BACK_VERTEX);
	IconRender::GetDevice()->SetTexture(0, NULL);
	IconRender::GetDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
	IconRender::GetDevice()->SetRenderState(D3DRS_AMBIENT, 0);

	IconRender::GetDevice()->DrawPrimitiveUP(D3DPT_TRIANGLELIST, 2, m_BackVertex, sizeof(IconObject::BACK_VERTEX));
*/
	//ACR`
	IconRender::GetDevice()->SetFVF(D3DFVF_ICON_VERTEX);
	IconRender::GetDevice()->SetTexture(0, m_pTexture->GetData());
	IconRender::GetDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
	//IconRender::GetDevice()->SetRenderState(D3DRS_AMBIENT, ((DWORD)m_LightColor[3]) && 0xffffff);
/*
	for (int i = 0; i < 3; ++i){
		D3DLIGHT9 light;
		ZeroMemory(&light, sizeof(D3DLIGHT9));
		light.Type = D3DLIGHT_DIRECTIONAL;
		light.Diffuse = m_LightColor[i] * 2.0f;
		light.Direction = m_LightDirection[i];
		IconRender::GetDevice()->SetLight(i, &light);
		IconRender::GetDevice()->LightEnable(i, TRUE);
	}
*/
	IconRender::GetDevice()->DrawPrimitiveUP(D3DPT_TRIANGLELIST, (UINT)(m_vecVertex.size() / 3), &m_vecVertex[0], sizeof(IconObject::ICON_VERTEX));
}

