#include "stdafx.h"
#include "afxinet.h"
#include "MemoryCardFtp.h"
#include "Log.h"
#include "Control.h"
#include "Resource.h"
#include "memccppDlg.h"
#include "Utility.h"

using namespace std;

const int iSleepTimeList = 10;

void MemoryCardFtp::SleepFtp(int ms)
{
	if (MustSleep()){
		Sleep(ms);
	}
}

bool MemoryCardFtp::MustSleep()
{
	return Control::GetDialog().GetCheckBoxBool(CHECK_BOX_WAIT_FTP);
}

MemoryCardFtp::MemoryCardFtp(const std::string& server_address)
{
	Log::Add("FTPڑJn");
	m_pFtpConnection = NULL;
	try{
		m_pFtpConnection = Control::GetInternetSession()->GetFtpConnection(server_address.c_str());
	} catch (CInternetException* pEx){
		Log::Add("G[: FTPڑł܂ł");
		TCHAR szErr[1024];
		if (pEx->GetErrorMessage(szErr, 1024)){
			//AfxMessageBox(szErr, MB_OK);
			Log::Add(szErr);
		} else {
			const string error_message = "FTPڑɕsȃG[܂";
			//AfxMessageBox(error_message.c_str(), MB_OK);
			Log::Add(error_message);
		}
		pEx->Delete();
		m_pFtpConnection = NULL;
		return;
	}

	m_iValidMemoryCardSlot = 0;
	m_pFtpConnection->SetCurrentDirectory("/mc");
	CInternetFile* list_file = m_pFtpConnection->Command("LIST", CFtpConnection::CmdRespRead, FTP_TRANSFER_TYPE_ASCII);
	SleepFtp(iSleepTimeList);
	if (list_file){
		Log::Add("FTP : MemoryCardFtp::EnumerateMemoryCard() Xgp[X");
		CString buffer;
		while (list_file->ReadString(buffer)){

			//Log::Add((LPCTSTR)buffer);

			FILE_DATA file_data;
			file_data.SetFtpListLine((LPCTSTR)buffer);
			if (file_data.name.find_first_not_of('.') == string::npos){
				continue;
			}
			m_iValidMemoryCardSlot |= (1 << Utility::ToInt(file_data.name));
		}
		list_file->Close();
		delete list_file;
		list_file = 0;
	} else {
		Log::Add("FTP : t@CXg̎擾Ɏs܂ MemoryCardFtp::EnumerateMemoryCard()");
		Log::LastError(GetLastError());
	}
}

MemoryCardFtp::~MemoryCardFtp()
{
	if (m_pFtpConnection){
		m_pFtpConnection->Close();
		delete m_pFtpConnection;
		m_pFtpConnection = NULL;
	} else {
		Log::Add("x : FTPڑɐؒf悤Ƃ܂");
	}
}

bool MemoryCardFtp::IsConnecting()
{
	return m_pFtpConnection != NULL;
}

bool MemoryCardFtp::ChangeDirectory(const std::string& directory_name)
{
//	const string dir = "/mc/" + Control::GetDialog().GetComboBoxString(COMBO_BOX_MEMORY_CARD) + "/" + directory_name + ((directory_name.empty()) ? "" : "/");
	const string dir = "/mc/" + Control::GetDialog().GetComboBoxString(COMBO_BOX_MEMORY_CARD) + "/" + directory_name;
	if (m_pFtpConnection->SetCurrentDirectory(dir.c_str())){
		m_strCurrectDirectory = directory_name;
	} else {
		Log::Add("FTP : JgfBNg̐ݒɎs܂");
		Log::LastError(GetLastError());
		return false;
	}
	return true;
}

const std::vector<MemoryCard::FILE_DATA> MemoryCardFtp::List()
{
	Log::Add("MemoryCardFtp::List()");
	ChangeDirectory(m_strCurrectDirectory);
	vector<MemoryCard::FILE_DATA> vec_res;
	vec_res.reserve(64);
	CInternetFile* res = m_pFtpConnection->Command("LIST", CFtpConnection::CmdRespRead, FTP_TRANSFER_TYPE_ASCII);
	SleepFtp(iSleepTimeList);
	if (res){
		Log::Add("MemoryCardFtp::List() Xgp[X");
		CString buffer;
		while (res->ReadString(buffer)){
			FILE_DATA file_data;
			file_data.SetFtpListLine((LPCTSTR)buffer);
			if (file_data.name.find_first_not_of('.') != string::npos && file_data.name.find(':') == string::npos){
				vec_res.push_back(file_data);
			}
			SleepFtp(iSleepTimeList);
		}
		res->Close();
		delete res;
		res = 0;
	} else {
		Log::Add("FTP : t@CXg̎擾Ɏs܂");
		Log::LastError(GetLastError());
	}
	return vec_res;
}

bool MemoryCardFtp::Remove(const std::string& file_name)
{
	Log::Add("MemoryCardFtp::Remove()");
	ChangeDirectory(m_strCurrectDirectory);
	if (!m_pFtpConnection->Remove(file_name.c_str())){
		Log::Add("FTP : t@C̍폜Ɏs܂ " + file_name);
		Log::LastError(GetLastError());
		return false;
	}
	return true;
}

const iBufferSize = 2048;
bool MemoryCardFtp::Put(const std::string& file_name)
{
	Log::Add("MemoryCardFtp::Put()");
	ChangeDirectory(m_strCurrectDirectory);
	if (MustSleep()){
		CInternetFile* file_to = m_pFtpConnection->OpenFile(file_name.c_str(), GENERIC_WRITE);
		if (file_to == NULL){
			Log::Add("FTP : t@C̃Abv[hɎs܂(zXg) " + file_name);
			Log::LastError(GetLastError());
			return false;
		}
		CFile file_from;
		if (!file_from.Open(file_name.c_str(), CFile::modeRead | CFile::shareDenyNone | CFile::osSequentialScan)){
			Log::Add("FTP : t@C̃Abv[hɎs܂([J) " + file_name);
			Log::LastError(GetLastError());
			return false;
		}
		char buffer[iBufferSize];
		UINT read_size;
		while ((read_size = file_from.Read(buffer, iBufferSize)) != 0){
			file_to->Write(buffer, read_size);
			SleepFtp(40);
		}
		file_to->Close();
		delete file_to;
		file_to = 0;
	} else {
		if (!m_pFtpConnection->PutFile(file_name.c_str(), file_name.c_str())){
			Log::Add("FTP : t@C̃Abv[hɎs܂ " + file_name);
			Log::LastError(GetLastError());
			return false;
		}
	}
	return true;
}

bool MemoryCardFtp::Get(const std::string& file_name)
{
	Log::Add("MemoryCardFtp::Get()");
	ChangeDirectory(m_strCurrectDirectory);
	if (!m_pFtpConnection->GetFile(file_name.c_str(), file_name.c_str())){
		Log::LastError(GetLastError());
		Log::Add("FTP : t@C̃_E[hɎs܂ " + file_name);
		return false;
	}
	return true;
}

bool MemoryCardFtp::RemoveDirectory(const std::string& directory_name)
{
	Log::Add("MemoryCardFtp::RemoveDirectory()");
	ChangeDirectory(m_strCurrectDirectory);
	if (!m_pFtpConnection->RemoveDirectory(directory_name.c_str())){
		Log::Add("FTP : fBNg̍폜Ɏs܂ " + directory_name);
		Log::LastError(GetLastError());
		return false;
	}
	return true;
}

bool MemoryCardFtp::MakeDirectory(const std::string& directory_name)
{
	Log::Add("MemoryCardFtp::MakeDirectory()");
	ChangeDirectory(m_strCurrectDirectory);
	if (!m_pFtpConnection->CreateDirectory(directory_name.c_str())){
		Log::Add("FTP : fBNg̍쐬Ɏs܂ " + directory_name);
		Log::LastError(GetLastError());
		return false;
	}
	return true;
}
