#include "stdafx.h"
#include <algorithm>
#include "PS1SaveData.h"
#include "DataConvert.h"

using namespace std;

struct PS1_SAVE_DATA_FILE_STRUCT{
	char magic[2];
	char type[2];
	char name[92];
	int16 palette[16];
	char pixel[3][128];
};

PS1SaveData::PS1SaveData(const std::string& file_path)
{
	m_bIsOpen = false;
	VERIFY(sizeof(PS1_SAVE_DATA_FILE_STRUCT) == 512);
	CFile file;
	if (!file.Open(file_path.c_str(), CFile::modeRead | CFile::shareDenyNone)){
		Log::Add("Z[uf[^t@CJ܂ł PS1SaveData");
		return;
	}
	if (file.GetLength() != 8192){
		Log::Add("Z[uf[^t@C̃TCYႢ܂ PS1SaveData");
		return;
	}
	PS1_SAVE_DATA_FILE_STRUCT save_data;
	if (file.Read(&save_data, sizeof(save_data)) != sizeof(save_data)){
		Log::Add("Z[uf[^t@Cǂݍ߂܂ł PS1SaveData");
		return;
	}
	char *c = save_data.magic;
	if (!((c[0] == 'S' || c[0] == 's') && (c[1] == 'C' || c[1] == 'c'))){
		Log::Add("Z[uf[^t@C̃}WbNio[Ⴂ܂ PS1SaveData");
		return;
	}
	char name_buffer[128];
	ZeroMemory(name_buffer, sizeof(name_buffer));
	strncpy(name_buffer, save_data.name, sizeof(save_data.name));
	m_strSaveDataName = name_buffer;
	m_vecTextureData.reserve(16 * 16 * 4);
	char palette[16][4];//ARGB
	for (int i = 0; i < 16; ++i){
		palette[i][0] = 0xff;
		for (int j = 0; j < 3; ++j){
			int bit_shift = j * 5;
			float f = (float)((((0x1f << bit_shift) & save_data.palette[i]) >> bit_shift) << 3);
			f *= (float)(0x100) / (float)(0xf8);
			palette[i][j + 1] = (char)std::min(f, 255.5f);
		}
	}
	for (int i = 0; i < 256; ++i){
		int index = i / 2;
		int bit_shift = (i & 1) * 4;
		int palette_index = (save_data.pixel[0][index] & ((char)0xf << bit_shift)) >> bit_shift;
		for (int j = 0; j < 4; ++j){
			m_vecTextureData.push_back(palette[palette_index][3 - j]);
		}
	}
	m_bIsOpen = true;
}
